# vim: noexpandtab fileencoding=utf-8
# Copyright © 2008 Sabin Iacob (m0n5t3r) <iacobs-gs@m0n5t3r.info>
# 
# This file is part of gScrobbler.
#
# gScrobbler is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import gpod, struct, dbus, os
from utils import log, chunkify
from time import mktime, localtime

__all__ = ('mount', 'umount', 'discover', 'list_recent')

class TimeManager:
	def __init__(self):
		self.plays = []
	
	def _intervals(self):
		p = self.plays
		for x in range(len(p) - 1):
			yield (x+1, p[x][0] - p[x+1][0] - p[x+1][1],)
	
	intervals = property(_intervals)

	def add_play(self, start, length):
		self.plays.append((start, length))
		self.plays.sort(lambda x,y: int(y[0] - x[0]))
	
	def alloc_int(self, nl, before):
		for i, s in self.intervals:
			if nl > s: continue
			ns = self.plays[i][0] + self.plays[i][1]
			self.add_play(ns, nl)
			return ns
		
		log(_('Couldn\'t find suitable interval between songs, adding to the start of the list...'), 'warning')

		ns = self.plays[-1][0] - 1
		self.add_play(ns, nl)
		return ns


bus = dbus.SystemBus()

def get_device(udi, interface = 'org.freedesktop.Hal.Device'):
	"""given an UDI and an optional interface name, return dbus interface
	
	>>> type(get_device('/org/freedesktop/Hal/devices/computer'))
	<class 'dbus.proxies.Interface'>
	"""
	global bus
	dev = bus.get_object('org.freedesktop.Hal', udi) 
	return dbus.Interface(dev, interface)

def mount(ipod):
	"""tell Hal to mount the device; ipod is an org.freedesktop.Hal.Device instance or an UDI"""
	if isinstance(ipod, basestring):
		ipod = get_device(ipod)
	
	if ipod.GetProperty('volume.is_mounted'):
		return ipod.GetProperty('volume.mount_point')
	
	mp = ipod.GetProperty('volume.label')
	fs = ipod.GetProperty('volume.fstype')
	opts = ['shortname=winnt', 'uid=%d' % os.getuid()]
	
	vol = dbus.Interface(ipod, 'org.freedesktop.Hal.Device.Volume')
	ret = vol.Mount(mp, fs, opts)
	
	if ret == 0:
		return ipod.GetProperty('volume.mount_point')

	return None

def umount(ipod):
	"""tell Hal to unmount the device; ipod is an org.freedesktop.Hal.Device instance or an UDI"""
	if isinstance(ipod, basestring):
		if ipod.startswith('/org/freedesktop/Hal/devices/volume') and not os.path.exists(ipod): #udi?
			ipod = get_device(ipod)
		else: # mount point?
			udi = resolve_udi(ipod)
			if udi is None: # no volume mounted there
				return True
			ipod = get_device(udi)
	
	if not ipod.GetProperty('volume.is_mounted'):
		return True
	
	vol = dbus.Interface(ipod, 'org.freedesktop.Hal.Device.Volume')
	return vol.Unmount([]) == 0

def resolve_mountpoint(udi):
	"""given an UDI, return its mountpoint or None if n/a
	
	>>> u = resolve_udi('/')
	>>> resolve_mountpoint(u)
	'/'

	print resolve_mountpoint('/')
	None
	"""
	dev = get_device(udi)
	try:
		return str(dev.GetProperty('volume.mount_point'))
	except dbus.DBusException:
		return None

def resolve_udi(mountpoint):
	"""given a filesystem path, find the device that is mounted there, return its UDI or None if not found
	
	>>> u = resolve_udi('/')
	>>> resolve_mountpoint(u)
	'/'

	>>> print resolve_udi(u)
	None
	"""
	hal = get_device('/org/freedesktop/Hal/Manager', 'org.freedesktop.Hal.Manager')
	mps = hal.FindDeviceStringMatch('volume.mount_point', mountpoint)
	if mps:
		return str(mps[0])
	return None

def discover(automount = True):
	"""
	connect to Hal and get a list of volumes with info.product = IPOD
	if automount is True, attemot to mount them (via Hal)
	return a list of mountpoints
	"""
	hal = get_device('/org/freedesktop/Hal/Manager', 'org.freedesktop.Hal.Manager')
	ipods = hal.FindDeviceStringMatch('info.product', 'IPOD')
	mountpoints = []
	for ipod in ipods:
		if automount:
			mp = mount(ipod)
		else:
			mp = resolve_mountpoint(ipod)
		
		#mp may be empty if volume is not mounted
		if mp:
			mountpoints.append(mp)
	
	return map(str, mountpoints)

def read_play_counts(mp):
	"""read play counts file, itunes database, do paranoid integrity checks"""

	itdb = gpod.itdb_parse(mp, None)

	assert itdb is not None, _('Couldn\'t find iTunes database under %s') % mp
	
	pcnt = os.path.join(os.path.dirname(itdb.filename), 'Play Counts')
	
	assert os.path.exists(pcnt), _('Play Counts file not found under %s') % mp

	pcfile = open(pcnt, 'r').read()
	
	hid, hlen, ilen, nent = struct.unpack('4s3I', pcfile[:16])
	ntracks = gpod.sw_get_list_len(itdb.tracks)
	pcsize = hlen + ilen * nent

	# a few more safety checks
	assert hid == 'mhdp', _('Unknown format (%(fmt)s) for play counts file %(fn)s') % {'fmt': hid, 'fn': pcnt}
	assert nent == ntracks, _('Track counts do not match (iTunes DB has %(it)d, Play Counts has %(pc)d)') % {'it': ntracks, 'pc': nent}
	assert len(pcfile) == pcsize, _('Incorrect size %(pc)d for play counts file %(fn)s, it needs to be %(sz)d') % {'pc': len(pcfile), 'fn': pcnt, 'sz': pcsize}

	return (itdb, pcnt, pcfile, hlen, ilen, nent)

def parse_play_counts(mp):
	"""extract relevant information from play counts, pass it to the calling function"""
	itdb, pcnt, pcfile, hlen, ilen, nent = read_play_counts(mp)

	cnts = map(lambda x: struct.unpack('7I', x), chunkify(pcfile, ilen, hlen))
	
	#filter, fix time by substracting 2082848400 from it (mac time starts in 1904, unix time starts in 1970)
	#retain play count, last play time, and rating
	return itdb, ((i, (cnts[i][0], cnts[i][1] - 2082844800, cnts[i][3])) for i in range(nent) if cnts[i][0])

def list_recent(mp):
	"""list recently played tracks on the iPod mounted at mp"""

	try:
		itdb, cnts = parse_play_counts(mp)
	except AssertionError, e:
		log(e, 'warning')
		return []
	except IOError, e:
		log(e, 'warning')
		return []

	songs = []
	extra = []
	t_man = TimeManager()

	for trackno, pc in cnts:
		t = gpod.sw_get_track(itdb.tracks, trackno)
		s = {
			'artist': t.artist,
			'title': t.title,
			'album': t.album,
			'trackno': t.track_nr,
			'rating': int(pc[2] / 20),
			'length': int(t.tracklen / 1000),
			'last_played': mktime(localtime(pc[1] - t.tracklen/1000)),
			'play_count': pc[0],
			'bogus_time': False
		}
		t_man.add_play(s['last_played'], s['length'])
		songs.append(s.copy())

		if s['play_count'] > 1:
			extra += (s['play_count'] - 1) * [s.copy()]
	
	for s in extra:
		s.update(bogus_time = True, last_played = t_man.alloc_int(s['length'], s['last_played']))
		songs.append(s.copy())

	# sort by last played timestamp
	songs.sort(lambda s1, s2: int(s1['last_played'] - s2['last_played']))
	return songs

def clear_play_counts(mp):
	"""back up play counts, reset them afterwards"""
	
	try:
		itdb, pcnt, pcfile, hlen, ilen, nent = read_play_counts(mp)
	except AssertionError, e:
		log(e, 'warning')
		return
	
	repl = struct.pack('II', 0, 0)
	chunks = (repl + c[8:] for c in chunkify(pcfile, ilen, hlen))

	pcfile = pcfile[:hlen] + ''.join(chunks)

	os.rename(pcnt, pcnt + '.bak')
	pc = open(pcnt, 'w+')
	pc.write(pcfile)
	pc.flush()
	pc.close()

