# vim: noexpandtab fileencoding=utf-8
# Copyright © 2008 Sabin Iacob (m0n5t3r) <iacobs-gs@m0n5t3r.info>
# 
# This file is part of gScrobbler.
#
# gScrobbler is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys
from urllib import urlopen, urlencode
from time import sleep

from config import settings
from utils import gmtimestamp, authtoken, log

__all__ = ('submit',)

failures = 0
interval = 1
session = None
now_playing_url = None
submit_url = None

def handshake():
	"""perform AudioScrobbler handshake"""
	global failures, interval, session, now_playing_url, submit_url
	
	log(_('starting handshake'), 'debug')
	log(_('failures: %(failures)d, interval: %(interval)d') % {'failures': failures, 'interval': interval}, 'debug')
	
	ts = gmtimestamp()

	hs_params = {
		'hs': 'true',
		'p': '1.2',
		'c': 'gsc',
		'v': '0.1',
		'u': settings['last.fm']['username'],
		't': int(ts),
		'a': authtoken(settings['last.fm']['password'], ts)
	}

	hs_url = "http://post.audioscrobbler.com/?%s" % urlencode(hs_params)

	log(_('hs_url is %s') % hs_url, 'debug')

	while interval <= 7200:
		try:
			log(_('Loading handshake url'), 'debug')
			if settings['dry_run']:
				ret = ['OK', 'dry session', 'now_mumu', 'mumu post']
			else:
				ret = [l.strip() for l in urlopen(hs_url).readlines()]
			interval = 1
			failures = 0
		except IOError:
			if failures == 0:
				interval = 60
			else:
				interval *= 2
			failures += 1
			log(_('Hard failure in handshake; failures = %(failures)d, interval = %(interval)d') % {'failures': failures, 'interval': interval}, 'warning')
			continue
		
		log(_('got: %s') % '\n'.join(ret), 'debug')
		if ret[0] == 'OK':
			failures = 0
			interval = 1
			session, now_playing_url, submit_url = ret[1:]
			log(_('Handshake success, submit url is %(sub)s, now playing url is %(np)s, session is %(ses)s') % {'sub': submit_url, 'np': now_playing_url, 'ses': session}, 'notice')
			sleep(interval)
			return ret[1:]
		elif ret[0] == 'BANNED':
			raise ValueError(_('Client banned from Last.fm'))
		elif ret[0] == 'BADAUTH':
			raise ValueError(_('Incorrect credentials supplied!'))
		elif ret[0] == 'BADTIME':
			raise ValueError(_('Clock skew too large. Correct your time before retrying!'))
		elif ret[0].startswith('FAILED'):
			failures += 1
			log(_('Hard failure in handshake; failures = %(failures)d, interval = %(interval)d, reason given: %(reason)s') % {'failures': failures, 'interval': interval, 'reason': ret[0][7:]}, 'warning')
		else:
			log(_('Hard failure in handshake; failures = %(failures)d, interval = %(interval)d, unknown error: %(err)s') % {'failures': failures, 'interval': interval, 'err': '\n'.join(ret)}, 'warning')
			failures += 1

		if failures == 1:
			interval = 60
		else:
			interval *= 2

		sleep(interval)

	exit(-1)
	
def as_send(data):
	global failures, interval, session, now_playing_url, submit_url
	log(_('failures: %d') % failures, 'debug')
	if not session:
		log(_('No session, starting handshake'), 'notice')
		try:
			handshake()
		except ValueError, e:
			log(e, 'error')
			exit(-1)
	
	data_form = [('s', session),]

	log(_('Got %d songs to submit') % len(data), 'notice')

	for s in data:
		idx = data.index(s)
		data_form += [
			('a[%d]' % idx, s['artist']),
			('t[%d]' % idx, s['title']),
			('i[%d]' % idx, int(gmtimestamp(s['last_played'], '%Y-%m-%d %H:%M:%S'))),
			('o[%d]' % idx, 'P'),
			('r[%d]' % idx, ('', 'L')[s['rating'] > 3]),
			('l[%d]' % idx, s['length']),
			('b[%d]' % idx, s['album']),
			('n[%d]' % idx, s['trackno'] or ''),
			('m[%d]' % idx, ''),
		]
	
	log(_('Data to post: %s') % data_form, 'debug')
	data = urlencode(data_form)

	log(_('Data to post (encoded): %s') % data, 'debug')
	
	try:
		log(_('Posting data to %s') % submit_url, 'debug')
		if settings['dry_run']:
			ret = 'OK'
		else:
			ret = urlopen(submit_url, data).read().strip()
	except IOError:
		log(_('Submission failed'), 'warning')
		sleep(interval)
		return False
	
	log(_('Response was "%s"') % ret, 'debug')

	sleep(interval)
	
	return ret == 'OK'

def submit(buf):
	global failures, session

	while True:
		if failures == 3:
			log(_('3 consecutive hard failures, resetting session and falling back to handshake'), 'warning')
			session = None
		if as_send(buf):
			failures = 0
			log(_('Submission success'), 'notice')
			break
		failures += 1
		log(_('Submit failed at attempt %d') % failures, 'warning')

