# vim: noexpandtab fileencoding=utf-8
# Copyright © 2008 Sabin Iacob (m0n5t3r) <iacobs-gs@m0n5t3r.info>
# 
# This file is part of gScrobbler.
#
# gScrobbler is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from md5 import md5 as _md5
from time import mktime, gmtime as _gmtime, strptime, struct_time
from gettext import translation
import sys, linecache, re

__all__ = ('md5', 'gmtime', 'gmtimestamp', 'authtoken', 'log', 'chunkify', 'traceit', 'uniq')

at_re = re.compile(r'[0-9a-fA-F]{32}')

def md5(str):
	"""md5 function to simplify hash generation
	
	>>> md5('boo')
	'ae3e83e2fab3a7d8683d8eefabd1e74d'
	"""
	return _md5(str).hexdigest()

def gmtime(t = None, fmt = None):
	"""gmtime as it should be; it can take:
	
	* floats
	>>> gmtime(1202503207.0)
	(2008, 2, 8, 20, 40, 7, 4, 39, 0)
	
	* integers
	>>> gmtime(1202503207)
	(2008, 2, 8, 20, 40, 7, 4, 39, 0)
	
	* tuples
	>>> gmtime((2008, 2, 8, 20, 40, 7, 4, 39, 0))
	(2008, 2, 8, 18, 40, 7, 4, 39, 0)
	
	* time tuples
	>>> from time import struct_time
	>>> gmtime(struct_time((2008, 2, 8, 20, 40, 7, 4, 39, 0)))
	(2008, 2, 8, 18, 40, 7, 4, 39, 0)
	
	* strings + strptime formats
	>>> gmtime('2008-02-08 00:03:12', '%Y-%m-%d %H:%M:%S')
	(2008, 2, 7, 22, 3, 12, 3, 38, 0)
	
	* returns standard struct_time
	>>> type(gmtime())
	<type 'time.struct_time'>

	>>> gmtime((2008, 2, 7, 22, 3, 12, 3, 38))
	Traceback (most recent call last):
	  File "<stdin>", line 1, in <module>
	  File "utils.py", line 39, in gmtime
	    raise TypeError('The sequence length must be 9, not %d' % len(t))
	TypeError: The sequence length must be 9, not 8
	"""
	if isinstance(t, (tuple, struct_time)):
		if len(t) != 9:
			raise TypeError('The sequence length must be 9, not %d' % len(t))
		t = mktime(t)
	elif isinstance(t, basestring):
		t = mktime(strptime(t, fmt))
	
	return _gmtime(t)

def gmtimestamp(t = None, fmt=None):
	"""useful addition: return UTC timestamp; takes the same options as the above gmtime
	
	>>> gmtimestamp('2008-02-08 00:03:12', '%Y-%m-%d %H:%M:%S')
	1202414592.0

	>>> gmtimestamp((2008, 2, 7, 22, 3, 12, 3, 38, -1))
	1202407392.0
	
	>>> gmtimestamp((2008, 2, 7, 22, 3, 12, 3, 38))
	Traceback (most recent call last):
	  File "<stdin>", line 1, in <module>
	  File "utils.py", line 62, in gmtimestamp
	    return mktime(gmtime(t, fmt))
	  File "utils.py", line 51, in gmtime
	    raise TypeError('The sequence length must be 9, not %d' % len(t))
	TypeError: The sequence length must be 9, not 8
	"""
	return mktime(gmtime(t, fmt))

def authtoken(password, ts):
	"""auth token for AudioScrobbler protocol version 1.2
	
	>>> authtoken('fubar', 1202407392.0)
	'e069bb0a347ca08842b826df4aa7ffda'
	
	>>> authtoken('5185e8b8fd8a71fc80545e144f91faf2', 1202407392.0)
	'e069bb0a347ca08842b826df4aa7ffda'
	"""
	if re.match(at_re, password): #password looks like md5
		pwd = password
	else:
		pwd = md5(password)
	return md5('%s%d' % (pwd, ts))

def log(msg, level = 'notice', debug = None):
	"""simple configurable logging
	
	>>> log('fasfda', level = 'debug', debug = 3)
	debug: fasfda
	
	>>> log('fasfda', level = 'notice', debug = 3)
	notice: fasfda
	"""
	levels = {
		'debug': 3,
		'notice': 2,
		'warning': 1,
		'error': 0,
	}
	if debug is None:
		from config import settings
		if 'debug' in settings:
			debug = settings.as_int('debug')
		else:
			debug = 0
	
	if levels[level] <= debug:
		print '%s: %s' % (level, msg)

def chunkify(data, size, offset = 0):
	"""split sequence type data in chunks of size, starting at offset"""
	return (data[offset + i:offset + i + size] for i in range(0, len(data) - offset, size))

def traceit(frame, event, arg):
	"""trace Python code execution 
	to trace:
	import sys
	sys.settrace('traceit')
	to stop tracing:
	sys.settrace(None)
	stolen from http://www.dalkescientific.com/writings/diary/archive/2005/04/20/tracing_python_code.html :)
	"""
	if event == "line":
		lineno = frame.f_lineno
		filename = frame.f_globals["__file__"]
		if (filename.endswith(".pyc") or
			filename.endswith(".pyo")):
			filename = filename[:-1]
		name = frame.f_globals["__name__"]
		line = linecache.getline(filename, lineno)
		print "%s:%s: %s" % (name, lineno, line.rstrip())
	return traceit

def uniq(seq):
	"""retain only unique items in a sequence
	
	>>> uniq([1,2,3,4,1,2,1])
	[1, 2, 3, 4]

	>>> uniq('asdfghsfa')
	'adgfhs'

	>>> uniq(('a','s','d','f','g','h','s','f','a'))
	['a', 'd', 'g', 'f', 'h', 's']
	"""
	ret = {}
	for x in seq:
		ret[x] = 1
	ret = ret.keys()
	if isinstance(seq, basestring):
		ret = ''.join(ret)
	return ret

def install_translation(domain, localedir = None, unicode = True):
	"""reimplementing gettext.install to give me a short name for ngettext"""
	import __builtin__
	t = translation(domain, localedir, fallback = True)
	__builtin__.__dict__['_'] = (unicode and t.ugettext or t.gettext)
	__builtin__.__dict__['n_'] = (unicode and t.ungettext or t.ngettext)

