#!/usr/bin/env python
# vim: noexpandtab fileencoding=utf-8
from glob import glob
from os.path import join, basename, dirname, abspath, exists
import os, sys, re

from distutils import log
from distutils.command.build import build as _build
from setuptools.command.install import install as _install
from setuptools import setup, find_packages, findall, Command


file_test = lambda file: file.endswith('.py') or file == 'as-submit'


class make_messages(Command):
	"""build and update pot and po files"""
	description = 'make/update translation files'
	user_options = [
		('domain=', 'd', 'translation domain'),
		('lang=', 'l', 'language file to update/create'),
		('all', 'a', 'update pot and all languages found (default if no language given)'),
	]
	boolean_options = ['all']

	def initialize_options(self):
		self.domain = None
		self.lang = []
		self.all = False
		self.basedir = None
		self.localedir = None
		self.files = []
	
	def finalize_options(self):
		packages = self.distribution.packages and [join(*(p.split('.'))) for p in self.distribution.packages] or []
		modules = self.distribution.py_modules and ['%s.py' % join(*(m.split('.'))) for m in self.distribution.py_modules] or []
		scripts = self.distribution.scripts or []
		files = scripts + modules
		for p in packages:
			files += [f for f in findall(p) if f.endswith('.py')]

		self.files = files
		self.domain = self.distribution.get_name().lower()
		self.basedir = abspath(dirname(__file__))
		self.localedir = abspath(join(self.basedir, 'po'))
		if not self.lang or self.all:
			self.all = True
			if not exists(self.basedir):
				os.makedirs(self.basedir)
			self.lang = [l[:-3] for l in glob(join(self.localedir, '*.po'))]
		
		if type(self.lang) is not list:
			self.lang = [self.lang]
	
	cnt_re = re.compile(r'\r?\n\r?\n')
	fuz_re = re.compile(r'#,.*fuzzy')
	ntr_re = re.compile(r'msgstr ""\r?\n\r?\n')

	def msgmerge(self, pofile, potfile):
		if exists(pofile):
			cmd = 'msgmerge -q "%s" "%s"' % (pofile, potfile)
			self.announce(cmd, log.DEBUG)
			(stdin, stdout, stderr) = os.popen3(cmd, 'b')
			msgs = stdout.read()
			errors = stderr.read()
			if errors:
				self.announce('errors happened while running msgmerge:', log.ERROR)
				self.announce(errors, log.ERROR)
				raise Exception(errors)
				sys.exit(8)
			open(pofile, 'wb').write(msgs)
			if pofile.endswith('.po'):
				counts = [
					(len(re.findall(self.cnt_re, msgs, 'm')), '%d messages'),
					(len(re.findall(self.fuz_re, msgs, 'm')), '%d fuzzy'),
					(len(re.findall(self.ntr_re, msgs, 'm')), '%d untranslated')
				]
				msg = ['updated %s' % basename(pofile)[:-3]] + [f % c for c, f in counts if c]
				self.announce(', '.join(msg), log.INFO)
		else:
			self.copy_file(potfile, pofile)
			self.announce('new translation: %s' % basename(pofile)[:-3], log.INFO)

	def make_pot(self):
		tmp_potfile = potfile = join(self.localedir, '%s.pot' % self.domain)
		package_name = self.distribution.get_name()
		package_version = self.distribution.get_version()
		copyright_holder = self.distribution.get_author()

		if exists(potfile):
			tmp_potfile += '.tmp'
			if exists(tmp_potfile): os.unlink(tmp_potfile)

		self.announce('generating %s.pot' % self.domain, log.INFO)

		for file in self.files:
			self.announce('processing file %s in %s' % (basename(file), dirname(file)), log.DEBUG)
			cmd = 'xgettext %(header)s -d %(domain)s -L Python --keyword=n_:1,2 --keyword=_ --from-code UTF-8 -o - --copyright-holder="%(ch)s" --package-name="%(name)s" --package-version="%(ver)s" "%(file)s"'
			cmd = cmd % {
				'header': exists(potfile) and '--omit-header' or '',
				'domain': self.domain, 
				'ch': copyright_holder, 
				'name': package_name, 
				'ver': package_version, 
				'file': file,
			}
			self.announce(cmd, log.DEBUG)
			(stdin, stdout, stderr) = os.popen3(cmd, 'b')
			msgs = stdout.read()
			errors = stderr.read()
			if errors:
				self.announce('errors happened while running xgettext on %s' % file, log.ERROR)
				self.announce(errors, log.ERROR)
				raise Exception(errors)
				sys.exit(8)
			if msgs:
				open(tmp_potfile, 'ab').write(msgs)
		
		if exists(tmp_potfile):
			(stdin, stdout, stderr) = os.popen3('msguniq "%s"' % tmp_potfile, 'b')
			msgs = stdout.read()
			errors = stderr.read()
			if errors:
				self.announce('errors happened while running msguniq', log.ERROR)
				self.announce(errors, log.ERROR)
				raise Exception(errors)
				sys.exit(8)
			open(tmp_potfile, 'w').write(msgs)
		
		if exists(tmp_potfile) and tmp_potfile != potfile:
			self.msgmerge(potfile, tmp_potfile)
			os.unlink(tmp_potfile)

		return potfile

	def run(self):
		potfile = self.make_pot()
		for lang in self.lang:
			self.announce('processing language %s' % lang, log.INFO)
			pofile = join(self.basedir, '%s.po' % lang)
			self.msgmerge(pofile, potfile)		


class build(_build):
	def has_locale(self):
		return exists(abspath(join('.', 'po')))
	
	sub_commands = _build.sub_commands + [('build_locale', has_locale)]


class build_locale(Command):
	description = 'build locale data'

	def initialize_options(self):
		self.locale_dir = abspath(join(dirname(__file__), 'po'))
		self.c_locale_dir = None
		self.build_dir = None
		self.domain = self.distribution.get_name().lower()
		self.lang = []
	
	def finalize_options(self):
		self.set_undefined_options('build', ('build_base', 'build_dir'))
		self.c_locale_dir = join(self.build_dir, 'locale')
		self.lang = [(basename(l)[:-3], l) for l in glob(join(self.locale_dir, '*.po'))]
	
	def run(self):
		for l, pofile in self.lang:
			l_dir = join(self.c_locale_dir, l, 'LC_MESSAGES')
			if not exists(l_dir):
				os.makedirs(l_dir)
			mofile = join(l_dir, '%s.mo' % self.domain)
			self.announce('processing language %s' % l, log.INFO)
			cmd = 'msgfmt -o "%s" "%s"' % (mofile, pofile)
			self.announce(cmd, log.DEBUG)
			os.system(cmd)


class install_locale(Command):
	description = 'install locale files'

	def initialize_options(self):
		self.build_dir = None
		self.data_dir = None
	
	def finalize_options(self):
		self.set_undefined_options('build', ('build_base', 'build_dir'))
		self.set_undefined_options('install', ('install_data', 'data_dir'))
	
	def get_locales(self):
		mo_files = [l.split(os.path.sep) for l in glob(join(self.build_dir, 'locale', '*', 'LC_MESSAGES', '*.mo'))]
		mo_files = [(join(*l), join(*([''] + [self.data_dir, 'share'] + l[-4:]))) for l in mo_files]
		return mo_files

	def run(self):
		self.run_command('build_locale')
		for src, dest in self.get_locales():
			self.copy_file(src, dest)

class install(_install):
	def has_locale(self):
		return exists(abspath(join('.', 'po')))
	
	def run(self):
		_install.run(self)
	
	sub_commands = _install.sub_commands + [('install_locale', has_locale)]
	

if __name__ == '__main__':
	setup(name = "gScrobbler",
		version = "0.1.1",
		description = "iPod to AudioScrobbler bridge",
		author = "Sabin Iacob (m0n5t3r)",
		author_email = "iacobs-gs@m0n5t3r.info",
		license = "GPLV3",
		url = "https://launchpad.net/gscrobbler",
		scripts = ['as-submit'],
		packages = find_packages(),
		zip_safe = True,
		cmdclass = {
			'build': build,
			'build_locale': build_locale,
			'install': install,
			'install_locale': install_locale,
			'translate': make_messages,
		}
	)

